pro destripe, drift_thermal, drifts_all, flag_valid_scans, map_back=map_back, smallfield=smallfield, $
              size_deg=size_deg, file_scalars=file_scalars, ind_scans=ind_scans, ind_subscans=ind_subscans, $
              maxnoise=maxnoise, cut_start=cut_start, time_1=time_1, t_proc=t_proc


;;; computes residual linear baselines from the redundancy

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; copyright 2007-2010 H\'el\`ene Roussel
;;; roussel@iap.fr
;;; Institut d'Astrophysique de Paris - Universit\'e Pierre et Marie Curie, France

;;; download and documentation: http://www2.iap.fr/users/roussel/herschel/

;;; Scanamorphos is distributed under the terms of the GNU General Public License
;;; (http://www.gnu.org/licenses/gpl.html).

;;; July 2010: public release version 1

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



restore, file_scalars


if ((size_deg lt 1. and nscans le 20) or nt lt 50000L) then ns = 49 else ns = 0

drifts_base = fltarr(nt_save, nb)

cube_signal = fltarr(dimx_iter, dimy_iter, nb, nscans)
cube_weight = fltarr(dimx_iter, dimy_iter, nb, nscans)
cube_subscan = lonarr(dimx_iter, dimy_iter, nb, nscans)
dimxy = dimx_iter * dimy_iter
dimxynb = dimxy * nb
tab_nb = dimxy * lindgen(nb)

tab_scan = lonarr(ind_scans(1))
for i = 1L, nscans - 1 do tab_scan = [temporary(tab_scan), i + lonarr(ind_scans(i+1) - ind_scans(i))]

tab_subscan = lonarr(ind_subscans(1))
for i = 1L, nsubscans - 1 do tab_subscan = [temporary(tab_subscan), i + lonarr(ind_subscans(i+1) - ind_subscans(i))]
tab_subscan += 1L

weight = 1. / maxnoise^2.


;;; remove duplicate offset grid and exclude scan starts after PCAL flashes (for PACS):
if nscans ge 4 then cut_dest = cut_start else cut_dest = 0
prepare_proj_coarse, file_scalars=file_scalars, /onegrid, cut_start=cut_dest, $
   ind_subscans=ind_subscans, ind_scans=ind_scans



restore, file_signal

restore, file_projcoarse

restore, file_vscan

restore, file_times

restore, file_time_coarse

flag_nza_coarse = interpol(flag_nza, time_s, time_save)


for k = 0L, nproj - 1 do begin
   revind_0 = revind(wproj(k))
   revind_1 = revind(wproj(k)+1)
   cp = revind_1 - revind_0
   wp = revind(revind_0:revind_1-1)
   wb = wp / nt
   ws = tab_scan(wp mod nt)
   wbs = wb + nb * ws
   ind_bs = sort(wbs)
   wp = wp(ind_bs)
   wb = wb(ind_bs)
   ws = ws(ind_bs)
   wbs = wbs(ind_bs)
   wt = wp mod nt
   ind_bs = uniq(wbs)
   cbs = n_elements(ind_bs)

   weight_sample = weight(tab_scan(wp mod nt) + nscans * (wp / nt))

   tab_ind = [-1, ind_bs]
   tab_b = wb(ind_bs)
   tab_s = ws(ind_bs)
   for ibs = 0L, cbs - 1 do begin
      wplane = tab_ind(ibs) + 1 + lindgen(tab_ind(ibs+1) - tab_ind(ibs))
      vectpos_plane = wproj(k) + dimxy * tab_b(ibs) + dimxynb * tab_s(ibs)
      cube_weight(vectpos_plane) += total(weight_sample(wplane))
      cube_signal(vectpos_plane) += total(signal(wp(wplane)) * weight_sample(wplane))
      cube_subscan(vectpos_plane) = median(tab_subscan(wt(wplane)))
   endfor
endfor

wcov = where(cube_weight gt 0.)
cube_signal(wcov) = cube_signal(wcov) / cube_weight(wcov)
w = where(finite(weight) gt 0)
cube_weight = cube_weight / mean(weight(w))
w = where(cube_weight le 0.)
cube_signal(w) = !values.f_nan

prepare_proj_coarse, file_scalars=file_scalars



;;; exclude sources using the same mask as baselines.pro:

map_mask = fltarr(dimx_iter, dimy_iter)
nx = (dimx / fact) * fact
ny = (dimy / fact) * fact
map_mask(0:dimx/fact-1, 0:dimy/fact-1) = rebin(map_back(0:nx-1, 0:ny-1), dimx/fact, dimy/fact)
w = where(finite(map_mask) lt 1 and total(cube_weight(*, *, *, 0), 3) gt 0. $
   and total(cube_weight(*, *, *, 1), 3) gt 0., cw)
if cw ge 1 then begin
   wz_init = reform(rebin(w, cw, nb) + (tab_nb ## (1L + lonarr(cw))), cw*nb)
   for s = 0L, nscans - 1 do begin
      wz = wz_init + dimxynb * s
      cube_weight(wz) = 0.
      cube_subscan(wz) = 0.
   endfor
endif



;;; create maps for single scans,
;;; and build reference maps for even and odd scans, excluding potentially deviant scans:

map_scans = total(cube_signal * cube_weight, 3, /nan)
weight_scans = total(cube_weight, 3)
w = where(weight_scans gt 0.)
map_scans(w) /= weight_scans(w)
w = where(weight_scans le 0.)
map_scans(w) = !values.f_nan

flag_valid_scans = 1 + intarr(nscans)
if nscans gt 2 then repeat begin
   nexc = 0
   ind_valid = where(flag_valid_scans gt 0, c_valid)
   map_median = median(map_scans(*, *, ind_valid), dimension=3, /even)
   dev = sqrt(median((map_scans(*, *, ind_valid) - rebin(map_median, dimx_iter, dimy_iter, c_valid))^2.))
   if nscans gt 20 then dev *= sqrt(float(nscans) / 20.)
   for s = 0, nscans - 1 do if flag_valid_scans(s) gt 0 then begin
      dev_scan = median(abs(map_scans(*, *, s) - map_median))
      if dev_scan gt 4. * dev then begin
         print, '     excluding deviant scan ' + chain(s) + ' from reference maps: ndev = ' $
            + chain(dev_scan / dev, 1)
         flag_valid_scans(s) = 0
         nexc = 1
      endif
   endif
endrep until nexc eq 0
ind_even = where((indgen(nscans) mod 2) eq 0 and flag_valid_scans gt 0, n_even)
ind_odd = where((indgen(nscans) mod 2) eq 1 and flag_valid_scans gt 0., n_odd)
ind_valid = where(flag_valid_scans gt 0)



;;; mask edges of potentially deviant subscans:

nexc_subs = 0
cut_subscans = intarr(nsubscans)
if keyword_set(cut_start) then begin
   wcut = where(cut_start gt 0, cw_cut)
   for c = 0, cw_cut - 1 do begin
      subs = where(ind_subscans ge ind_scans(wcut(c)))
      cut_subscans(subs(0)) = 1
   endfor
endif

if nscans ge 6 then for subs = 0L, nsubscans - 1 do if cut_subscans(subs) lt 1 then begin
   w = where(ind_scans le ind_subscans(subs))
   scan = max(w)
   weight_subs = cube_weight(*, *, *, scan)
   w = where(cube_subscan(*, *, *, scan) ne subs + 1)
   weight_subs(w) = 0.
   map_subscan = total(cube_signal(*, *, *, scan) * weight_subs, 3, /nan)
   weight_subscan = total(weight_subs, 3)
   w = where(weight_subscan gt 0.)
   map_subscan(w) /= weight_subscan(w)
   w = where(weight_subscan le 0.)
   map_subscan(w) = !values.f_nan

   if (scan mod 2) eq 0 then ind_ref = where((indgen(nscans) mod 2) eq 0 and indgen(nscans) ne scan) $
      else ind_ref = where((indgen(nscans) mod 2) eq 1 and indgen(nscans) ne scan)
   map_ref = total(map_scans(*, *, ind_ref) * weight_scans(*, *, ind_ref), 3, /nan)
   weight_ref = total(weight_scans(*, *, ind_ref), 3)
   w = where(weight_ref gt 0.)
   map_ref(w) /= weight_ref(w)
   dev_subscan = median(abs(map_subscan - map_ref))
   if dev_subscan gt 8. * dev then begin
      print, '     excluding edges of deviant subscan ' + chain(subs) + ' from reference maps: ndev = ' $
         + chain(dev_subscan / dev, 1)
      nexc_subs = 1
      ind_start = (ind_subscans(max([subs-1, 0])) + ind_subscans(subs)) / 2
      ind_end = (ind_subscans(subs+1) + ind_subscans(min([subs+2, nsubscans]))) / 2
      nsubs = ind_subscans(subs+1) - ind_subscans(subs)
      w = where(flag_nza(ind_start:ind_end) eq 0, cw)
      if cw ge 1 then begin
         signal(ind_start+w, *) = !values.f_nan
         if (not keyword_set(x_bolos)) then restore, file_pixcoords
         for b = 0L, nb - 1 do $
            cube_weight(floor(x_bolos(ind_start+w, b) / fact), floor(y_bolos(ind_start+w, b) / fact), b, scan) = 0.
      endif
   endif

   x_bolos = 0
   y_bolos = 0
endif

if nexc_subs gt 0 then begin
   map_scans = total(cube_signal * cube_weight, 3, /nan)
   weight_scans = total(cube_weight, 3)
   w = where(weight_scans gt 0.)
   map_scans(w) /= weight_scans(w)
   w = where(weight_scans le 0.)
   map_scans(w) = !values.f_nan

   save, file=file_signal, signal, /xdr
   prepare_proj_fine, file_scalars=file_scalars, ind_scans=ind_scans
   prepare_proj_coarse, file_scalars=file_scalars
endif



if n_even ge 2 then begin
   ima_even = total(map_scans(*, *, ind_even) * weight_scans(*, *, ind_even), 3, /nan)
   weight_even = total(weight_scans(*, *, ind_even), 3)
endif else begin
   ima_even = map_scans(*, *, ind_even) * weight_scans(*, *, ind_even)
   weight_even = weight_scans(*, *, ind_even)
endelse
w = where(weight_even gt 0.)
ima_even(w) /= weight_even(w)
if n_odd ge 2 then begin
   ima_odd = total(map_scans(*, *, ind_odd) * weight_scans(*, *, ind_odd), 3, /nan)
   weight_odd = total(weight_scans(*, *, ind_odd), 3)
endif else begin
   ima_odd = map_scans(*, *, ind_odd) * weight_scans(*, *, ind_odd)
   weight_odd = weight_scans(*, *, ind_odd)
endelse
w = where(weight_odd gt 0.)
ima_odd(w) /= weight_odd(w)



;;; first destripe scan by scan:

for s_ref = 1, 0, -1 do begin

   if s_ref eq 1 then begin
      ima_weight = weight_odd
      ima_signal = ima_odd
   endif else begin
      ima_weight = weight_even
      ima_signal = ima_even
   endelse

   for s = 0, nscans - 1 do if ((s - s_ref) mod 2) ne 0 then begin
      tab_off = fltarr(nb)
      for b = 0L, nb - 1 do if finite(maxnoise(s, b)) gt 0 then begin
         w = where(cube_weight(*, *, b, s) * ima_weight gt 0., cw)
         if cw ge 1 then tab_off(b) += median(cube_signal(w+dimxy*b+dimxynb*s) - ima_signal(w), /even)
      endif

      signal(ind_scans(s):ind_scans(s+1)-1, *) -= $
         (tab_off ## (1. + fltarr(ind_scans(s+1) - ind_scans(s))))
      wsave = where(time_save ge time_s(ind_scans(s)) and time_save le time_s(ind_scans(s+1)-1), csave)
      drifts_base(wsave, *) += (tab_off ## (1. + fltarr(csave)))
      w = where(cube_weight(*, *, *, s) gt 0.)
      cube_s = cube_signal(*, *, *, s)
      wb = w / dimxy
      cube_s(w) -= tab_off(wb)
      cube_signal(*, *, *, s) = cube_s
   endif
endfor



;;; then destripe subscan by subscan:

weight_scans = total(cube_weight, 3)
ima_weight = total(weight_scans(*, *, ind_valid), 3)
if nscans lt 4 then begin
   wz = where(weight_even * weight_odd le 0.)
   ima_weight(wz) = 0.
endif


restore, file_pixcoords

indxy_coarse = floor(x_bolos / fact) + dimx_iter * floor(y_bolos / fact)
x_bolos = 0
y_bolos = 0

ima_diff = fltarr(dimx_iter, dimy_iter)
simul = fltarr(nt, nb)


w = where(ind_subscans lt ind_scans(1), cw)
subs_visu = cw / 2


nloop = 0
if inst eq 'spire' then begin
   min_nloop = 3
   max_nloop = 9
   noise_level = median(maxnoise)
endif else begin
   min_nloop = 2
   if (keyword_set(smallfield) and size_deg lt 1.) then max_nloop = 3 else max_nloop = 2
   noise_level = 3. * median(maxnoise)
endelse

amp_simul = fltarr(nsubscans)
amp_error = fltarr(nsubscans)
error_fit = fltarr(nsubscans, nb)
ind_excl = intarr(nsubscans)


repeat begin
   ima_signal = total(total(cube_signal(*, *, *, ind_valid) $
      * cube_weight(*, *, *, ind_valid), 4, /nan), 3, /nan)
   norm_weight = total(total(cube_weight(*, *, *, ind_valid), 4), 3)
   w = where(norm_weight gt 0.)
   ima_signal(w) /= norm_weight(w)

   for subs = 0L, nsubscans - 1 do begin
      s = where(ind_scans(0:nscans-1) le ind_subscans(subs) and ind_scans(1:*) gt ind_subscans(subs))
      s = s(0)

      nsubs = ind_subscans(subs+1) - ind_subscans(subs)
      indxy_subs = indxy_coarse(ind_subscans(subs):ind_subscans(subs+1)-1, *)
      simul_subs = fltarr(ind_subscans(subs+1)-ind_subscans(subs), nb)
      time_subs = time_s(ind_subscans(subs):ind_subscans(subs+1)-1)
      w = where(vscan(ind_subscans(subs):ind_subscans(subs+1)-1) gt 3.)
      indxy_fit = indxy_subs(w, *)
      time_fit = time_subs(w)

      for b = 0L, nb - 1 do if finite(maxnoise(s, b)) gt 0 then begin
         wxy = where(cube_subscan(*, *, b, s) eq subs + 1, cxy)
         if cxy ge 1 then begin
            w = where(ima_weight(wxy) gt 0., cw)
            if cw ge 10 then begin
               ima_diff += !values.f_nan
               ima_diff(wxy(w)) = cube_signal(wxy(w)+dimxy*b+dimxynb*s) - ima_signal(wxy(w))
               fit_subs = absdev_fit(time_fit, ima_diff(indxy_fit(*, b)), coef=coef, $
                  noise_level=noise_level, /sym, /dyn, ns=ns)
               werr = where(finite(ima_diff(indxy_fit(*, b))) gt 0)
               error_fit(subs, b) = sqrt(mean((ima_diff(indxy_fit(werr, b)) - fit_subs(werr))^2.))
               simul_subs(*, b) = coef(0) + coef(1) * time_subs
               if (keyword_set(debug) and subs eq subs_visu and b eq bref) then begin
                  time_2 = systime(/seconds)
                  t_proc += (time_2 - time_1)
                  wset, 0
                  w = where(finite(ima_diff(indxy_fit(*, b))) gt 0)
                  otherplot, time_fit(w), ima_diff(indxy_fit(w, b)), xtitle='time (s)', $
                     ytitle='signal (Jy/bolometer)', title='bolometer '+chain(bref) $
                     +', subscan '+chain(subs)+': observed - simulated from map', psym=-7
                  otherplot, /over, color=10, time_subs, simul_subs(*, b)
                  res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
                  time_1 = systime(/seconds)
               endif
               cube_signal(indxy_subs(*, b)+dimxy*b+dimxynb*s) -= simul_subs(*, b)
            endif
         endif
      endif

      simul(ind_subscans(subs):ind_subscans(subs+1)-1, *) += simul_subs
      amp_simul(subs) = median(sqrt(total((simul_subs - (median(simul_subs, dimension=1) ## (fltarr(nsubs) + 1.)))^2., 1) / float(nsubs)) / maxnoise(s, *), /even)
      amp_error(subs) = median(error_fit(subs, *) / maxnoise(s, *), /even)
   endfor

   amp_dev = amp_error
   for s = 0, nscans - 1 do begin
      wsubs = where(ind_subscans ge ind_scans(s) and ind_subscans lt ind_scans(s+1))
      med_amp = median(amp_error(wsubs), /even)
      amp_dev(wsubs) -= med_amp
   endfor
   sig_amp = sqrt(mean(amp_dev^2.))
   ndev = amp_dev / sig_amp
   w = where(ndev gt 4. and ind_excl lt 1, cw)
   if cw ge 1 then begin
      print, '     excluding deviant subscan(s) ' + chain(w) + ' from reference map: ndev = ' + chain(ndev(w), 1)
      ind_excl(w) = 1
      if keyword_set(debug) then begin
         time_2 = systime(/seconds)
         t_proc += (time_2 - time_1)
         wset, 0
         otherplot, lindgen(nsubscans), amp_dev, psym=4, xtitle='subscan index', ytitle='normalized error'
         otherplot, /over, lstyle=2, color=10, lindgen(nsubscans), fltarr(nsubscans) + 2. * sig_amp
         otherplot, /over, lstyle=2, color=10, lindgen(nsubscans), fltarr(nsubscans) + 3. * sig_amp
         otherplot, /over, lstyle=2, color=10, lindgen(nsubscans), fltarr(nsubscans) + 4. * sig_amp
         if cw ge 1 then otherplot, /over, color=10, w, amp_dev(w), psym=4
         res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
         time_1 = systime(/seconds)
      endif
   endif
   for ks = 0L, cw - 1L do begin
      subs = w(ks)
      wz = where(cube_subscan eq subs + 1)
      cube_weight(wz) = 0.
   endfor

   nloop += 1
   if keyword_set(debug) then print, mean(amp_error), mean(amp_simul)
endrep until ((nloop ge min_nloop and (mean(amp_error) le 1. or mean(amp_simul) le 0.1)) or nloop ge max_nloop)


for s = 0, nscans - 1 do begin
   simul_scan = simul(ind_scans(s):ind_scans(s+1)-1, *)
   wnza = where(flag_nza(ind_scans(s):ind_scans(s+1)-1) eq 0, cnza, complement=wdef, ncomplement=cdef)
   if cnza ge 1 then begin
      tab_simul = transpose([simul_scan(wdef(0), *), simul_scan(wdef, *), simul_scan(wdef(cdef-1), *)])
      tab_time = [time_s(ind_scans(s))-maxtimediff, time_s(ind_scans(s)+wdef), time_s(ind_scans(s+1)-1)+maxtimediff]
      ind_time = interpol(findgen(cdef+2), tab_time, time_s(ind_scans(s):ind_scans(s+1)-1))
      simul_scan = transpose(interpolate(tab_simul, ind_time))
   endif
   signal(ind_scans(s):ind_scans(s+1)-1, *) -= simul_scan
   wsave = where(time_save ge time_s(ind_scans(s)) and time_save le time_s(ind_scans(s+1)-1), csave)
   cscan = ind_scans(s+1) - ind_scans(s)
   tab_simul = transpose([simul_scan(0, *), simul_scan, simul_scan(cscan-1, *)])
   tab_time = [time_s(ind_scans(s))-maxtimediff, time_s(ind_scans(s):ind_scans(s+1)-1), time_s(ind_scans(s+1)-1)+maxtimediff]
   ind_time = interpol(findgen(cscan+2), tab_time, time_save(wsave))
   drifts_base(wsave, *) += transpose(interpolate(tab_simul, ind_time))
endfor



w = where(finite(signal) gt 0)
med = median(signal(w))
sig = sqrt(mean((signal(w) - med)^2.))
for k = 0, 9 do begin
   w = where(signal - med lt 3. * sig)
   med = median(signal(w))
   sig = sqrt(mean((signal(w) - med)^2.))
endfor
off = med - sig
signal -= off

drifts_all += drifts_base
for i = 0, nscans - 1 do begin
   wb = where(finite(maxnoise(i, *)) gt 0, cb)
   wsave = where(time_save ge time_s(ind_scans(i)) and time_save le time_s(ind_scans(i+1)-1), csave)
   drifts_scan = drifts_base(wsave, *)
   drift_thermal(wsave) += total(drifts_scan(*, wb) / rebin(maxnoise(i, wb), csave, cb)^2., 2) $
      / total(1./ maxnoise(i, wb)^2.)
endfor


save, file=file_signal, signal, /xdr

end
