pro stitch_blocks, dimx_final=dimx_final, dimy_final=dimy_final, nplanes=nplanes, nblocks=nblocks, $
    plane_weight=plane_weight, root_out=root_out, hdr=hdr


;;; for observations sliced into several blocks (sub-fields overlapping at the edges):
;;; combine all blocks into a simgle map

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; copyright 2007-2010 H\'el\`ene Roussel
;;; roussel@iap.fr
;;; Institut d'Astrophysique de Paris - Universit\'e Pierre et Marie Curie, France

;;; download and documentation: http://www2.iap.fr/users/roussel/herschel/

;;; Scanamorphos is distributed under the terms of the GNU General Public License
;;; (http://www.gnu.org/licenses/gpl.html).

;;; July 2010: public release version 1

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



dimxy = dimx_final * dimy_final

cube_combined = fltarr(dimx_final, dimy_final, nplanes)


for block = 0L, nblocks - 1 do begin
   cube = readfits(root_out + '_' + chain(block) + '.fits', /silent)

   ;;; determine region of new block to be combined with others (sufficient weight):
   map_weight = cube(*, *, plane_weight)
   map_weight_combined = cube_combined(*, *, plane_weight)
   w_comb = where(map_weight gt map_weight_combined)

   ;;; remove possible bias in signal plane between different blocks:
   if block gt 0 then begin
      w_over = where(map_weight * map_weight_combined gt 0.)
      m = min([max(map_weight(w_over)), max(map_weight_combined(w_over))])
      w_over = where(map_weight gt 2. * m / 3. and map_weight_combined gt 2. * m / 3.)
      off = total((cube(w_over) - cube_combined(w_over)) * map_weight(w_over) * map_weight_combined(w_over)) $
         / total(map_weight(w_over) * map_weight_combined(w_over))
      print, 'offset of block ' + chain(block) + ' wrt preceding block(s) = ' + chain(off, 5)
      cube_combined(w_comb) = cube(w_comb) - off
   endif else cube_combined(w_comb) = cube(w_comb)

   ;;; add weight and error planes with no bias:
   for k = 1, nplanes - 1 do cube_combined(w_comb+k*dimxy) = cube(w_comb+k*dimxy)
endfor

writefits, root_out + '_combined.fits', cube_combined, hdr


end
