function subtract_aver_drift, drift_thermal, drifts_all, flag_valid_scans=flag_valid_scans, maxnoise=maxnoise, $
         highfnoise=highfnoise, ind_scans=ind_scans, ind_subscans=ind_subscans, ind_aver=ind_aver, $
         file_scalars=file_scalars, title=title, min_nsamp=min_nsamp, detect_gl=detect_gl, $
         t_cross_fwhm=t_cross_fwhm, f_fwhm=f_fwhm, parallel=parallel, nocross=nocross, iter=iter, $
         cut_start=cut_start, time_1=time_1, t_proc=t_proc


;;; computes average drift, interpolates and optionally subtracts it from the signal

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; copyright 2007-2010 H\'el\`ene Roussel
;;; roussel@iap.fr
;;; Institut d'Astrophysique de Paris - Universit\'e Pierre et Marie Curie, France

;;; download and documentation: http://www2.iap.fr/users/roussel/herschel/

;;; Scanamorphos is distributed under the terms of the GNU General Public License
;;; (http://www.gnu.org/licenses/gpl.html).

;;; July 2010: public release version 1

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



restore, file_scalars

crs = string(13b) + string(10b)


compute_aver_drift, w_glitches, nb_glitch, maxnoise=maxnoise, highfnoise=highfnoise, $
   ind_scans=ind_scans, ind_aver=ind_aver, file_scalars=file_scalars, min_nsamp=min_nsamp, $
   detect_gl=detect_gl, t_cross_fwhm=t_cross_fwhm, f_fwhm=f_fwhm, parallel=parallel, $
   time_1=time_1, t_proc=t_proc


;;; mask glitches and update projection vectors:
if nb_glitch ge 1 then begin
   print, crs + 'removed ' + chain(nb_glitch) + ' sample(s) affected by glitches'
   restore, file_signal
   signal(w_glitches) = !values.f_nan
   save, file=file_signal, signal, /xdr
   signal = 0

   prepare_proj_coarse, file_scalars=file_scalars
   prepare_proj_fine, file_scalars=file_scalars, ind_scans=ind_scans
   w_glitches = 0

   if (keyword_set(visu) and nb_glitch gt 100) then begin
      time_2 = systime(/seconds)
      t_proc += (time_2 - time_1)
      restore, file_maps
      proj_map, map_noglitch, file_scalars=file_scalars, maxnoise=maxnoise, ind_scans=ind_scans
      disp_ima, map_signal - map_noglitch, win=4, title='map of residual glitches', $
         weightmap=map_weight, /trunc_max
      res = dialog_message('Continue and delete window ?', /center, /information, title='Pause')
      wdelete, 4
      time_1 = systime(/seconds)
   endif
endif



restore, file_times

restore, file_time_coarse


drifts = fltarr(nt)
amp_drifts = fltarr(nscans, nb)



;;; reconstruct average absolute drift from drifts difference pairs:

reconstruct_aver_drift, drifts_save, npairs_save, file_scalars=file_scalars, ind_scans=ind_scans, $
   ind_subscans=ind_subscans, noise_level=median(maxnoise), time_1=time_1, t_proc=t_proc


npairs_min = median(npass_crit / (2. * (maxnoise / median(maxnoise))^2.))


;;; cancel correction if it is too sparsely determined:

flag_nza_coarse = interpol(flag_nza, time_s, time_save)
wnom = where(flag_nza_coarse gt 0, cnom)
wdef = where(flag_nza_coarse gt 0 and npairs_save gt npairs_min, cdef)
if cdef lt 0.7 * cnom then begin
   print, '              average drift too sparsely determined -> correction cancelled.'
   amp_drifts = fltarr(nscans, nb)
   return, amp_drifts
endif


wdef = where(npairs_save gt npairs_min)
med = mean(drifts_save(wdef))


;;; suppress potential spikes in average drift (supposed to be smooth):
drifts_smooth = median(drifts_save(wdef), 11)
w = where(abs(drifts_save(wdef) - drifts_smooth) gt 2. * median(maxnoise), cw)
if cw ge 1 then npairs_save(wdef(w)) = 0.


for i = 0, nscans - 1 do begin
   wsave = where(time_save ge time_s(ind_scans(i)) and time_save le time_s(ind_scans(i+1)-1), csave)
   wdef = where(time_save ge time_s(ind_scans(i)) and time_save le time_s(ind_scans(i+1)-1) $
      and npairs_save gt npairs_min, cdef)
   wz = where(time_save ge time_s(ind_scans(i)) and time_save le time_s(ind_scans(i+1)-1) $
      and npairs_save le npairs_min, cz)
   if cdef ge 10 then begin
      if cz ge 1 then begin
         tab_time = [time_s(ind_scans(i)) - timestep, time_save(wdef), time_s(ind_scans(i+1)-1) + timestep]
         tab_drifts = [drifts_save(wdef(0)), median(drifts_save(wdef), 7), drifts_save(wdef(cdef-1))]
         drifts_save(wz) = interpol(tab_drifts, tab_time, time_save(wz))
      endif
      tab_time = [time_s(ind_scans(i)) - timestep, time_save(wsave), time_s(ind_scans(i+1)-1) + timestep]
      tab_drifts = [drifts_save(wsave(0)), drifts_save(wsave), drifts_save(wsave(csave-1))]
      subs_drifts = interpol(tab_drifts, tab_time, time_s(ind_scans(i):ind_scans(i+1)-1))
      drifts(ind_scans(i):ind_scans(i+1)-1) = subs_drifts
      dev = sqrt(mean((subs_drifts - med)^2.))
      amp_drifts(i, *) = 3. * dev
   endif else drifts_save(wsave) = 0.
endfor

;;; remove zero level of drifts:
off = median(drifts)
drifts -= off
drifts_save -= off




;;; remove drift common to all cross-scans
;;; (solution to the problem of deriving the absolute drift from the drift differences matrix
;;; is not unique, and is affected by oscillations with a periodicity of 2 scans, i.e. 1 cross-scan,
;;; that can be removed in this way):

if (nscans ge 2 and (not keyword_set(nocross))) then begin
   restore, file_signal
   signal_save = signal
   restore, file_maps
   map_signal_save = map_signal
   map_weight_save = map_weight

   map_drift = fltarr(dimx, dimy, nscans)
   weight_drift = fltarr(dimx, dimy, nscans)
   signal = rebin(drifts, nt, nb)
   save, file=file_signal, signal, /xdr
   simnoise = 1. + fltarr(nscans, nb)
   for i = 0, nscans - 1 do begin
      prepare_proj_fine, file_scalars=file_scalars, scan_next=i+1, ind_scans=ind_scans
      proj_map, file_scalars=file_scalars, maxnoise=simnoise, ind_scans=ind_scans
      restore, file_maps
      map_drift(*, *, i) = map_signal
      weight_drift(*, *, i) = map_weight
   endfor
   w = where(weight_drift le 0.)
   map_drift(w) = !values.f_nan


   if nscans lt 8 then begin
      width = 30
      sq_width = width^2.
      tab_width = [width, width, 1]
      map_sq = map_drift^2.
      map_smooth = smooth(map_drift, tab_width, /nan, /edge_truncate)
      map_smooth = (map_smooth * sq_width - map_drift) / (sq_width - 1.)
      map_smoothsq = smooth(map_sq, tab_width, /nan, /edge_truncate)
      map_smoothsq = (map_smoothsq * sq_width - map_sq) / (sq_width - 1.)
      map_dev2 = map_smoothsq - map_smooth^2.

      if (max(cut_start) gt 0 and nscans ge 4) then begin
         wcut = where(cut_start gt 0, cw_cut)
         map_dev2(*, *, wcut) = !values.f_nan
      endif

      map_ref = total(map_drift / map_dev2, 3, /nan) / total(1. / map_dev2, 3, /nan)
      if nscans lt 4 then begin
         w = where(weight_drift(*, *, 0) le 0. or weight_drift(*, *, 1) le 0.)
         map_ref(w) = 0.
      endif
   endif


   if keyword_set(debug) then begin
      time_2 = systime(/seconds)
      t_proc += (time_2 - time_1)
      w = where(finite(map_drift) gt 0, cw)
      ind_sort = sort(map_drift(w))
      min_d = map_drift(w(ind_sort(100)))
      max_d = map_drift(w(ind_sort(cw-101)))
      disp_ima, win=10, map_drift(*, *, 0), weightmap=weight_drift(*, *, 0), $
         title='mapping of scan 0 average drift', min_map=min_d, max_map=max_d, noise_level=0.01, pow_int=0.5
      disp_ima, win=11, map_drift(*, *, 1), weightmap=weight_drift(*, *, 1), $
         title='mapping of scan 1 average drift', min_map=min_d, max_map=max_d, noise_level=0.01, pow_int=0.5
      if nscans lt 8 then begin
         weight_ref = fltarr(dimx, dimy)
         w = where(weight_drift(*, *, 0) gt 0. or weight_drift(*, *, 1) gt 0.)
         weight_ref(w) = 10.
         disp_ima, win=9, map_ref, weightmap=weight_ref, title='reference drift map', $
            min_map=min_d, max_map=max_d, noise_level=0.01, pow_int=0.5
      endif
      time_1 = systime(/seconds)
   endif



   dimxy = dimx * dimy
   restore, file_pixcoords
   nt_scan_even = ind_scans(1) - ind_scans(0)
   simul_scan_even = fltarr(nt_scan_even, nscans)
   simul_scan_even(*, 0) = drifts(ind_scans(0):ind_scans(1)-1)
   nt_scan_odd = ind_scans(2) - ind_scans(1)
   simul_scan_odd = fltarr(nt_scan_odd, nscans)
   simul_scan_odd(*, 1) = drifts(ind_scans(1):ind_scans(2)-1)


   indxy_bref = floor(x_bolos(*, bref)) + dimx * floor(y_bolos(*, bref))
   if nscans lt 8 then begin
      simul_ref_even = map_ref(indxy_bref(ind_scans(0):ind_scans(1)-1))
      simul_ref_odd = map_ref(indxy_bref(ind_scans(1):ind_scans(2)-1))
   endif
   for i = 1, nscans - 1 do $
      simul_scan_even(*, i) = map_drift(indxy_bref(ind_scans(0):ind_scans(1)-1) + dimxy * i)
   for i = 0, nscans - 1 do if i ne 1 then $
      simul_scan_odd(*, i) = map_drift(indxy_bref(ind_scans(1):ind_scans(2)-1) + dimxy * i)


   if nscans lt 8 then begin
      simul_excess_even = min(simul_scan_even - rebin(simul_ref_even, nt_scan_even, nscans), $
         dimension=2, /absolute, /nan) + simul_ref_even
      simul_excess_odd = min(simul_scan_odd - rebin(simul_ref_odd, nt_scan_odd, nscans), $
         dimension=2, /absolute, /nan) + simul_ref_odd
   endif else begin
      simul_excess_even = median(simul_scan_even, dimension=2, /even)
      simul_excess_odd = median(simul_scan_odd, dimension=2, /even)
   endelse


   if keyword_set(debug) then begin
      time_2 = systime(/seconds)
      t_proc += (time_2 - time_1)
      wset, 0
      otherplot, time_s(ind_scans(0):ind_scans(1)-1), simul_scan_even(*, 0), title='even scan direction'
      w = where(finite(simul_scan_even(*, 1)) gt 0)
      otherplot, /over, color=10, time_s(ind_scans(0)+w), simul_scan_even(w, 1)
      if nscans lt 8 then begin
         w = where(finite(simul_ref_even) gt 0)
         otherplot, /over, color=200, time_s(ind_scans(0)+w), simul_ref_even(w)
      endif
      res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
      otherplot, time_s(ind_scans(1):ind_scans(2)-1), simul_scan_odd(*, 1), title='odd scan direction'
      w = where(finite(simul_scan_odd(*, 0)) gt 0)
      otherplot, /over, color=10,time_s(ind_scans(1)+w), simul_scan_odd(w, 0)
      if nscans lt 8 then begin
         w = where(finite(simul_ref_odd) gt 0)
         otherplot, /over, color=200, time_s(ind_scans(1)+w), simul_ref_odd(w)
      endif
      res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
      time_1 = systime(/seconds)
   endif


   simul_aver = fltarr(nt)
   simul_aver(ind_scans(0):ind_scans(1)-1) = simul_excess_even
   simul_aver(ind_scans(1):ind_scans(2)-1) = simul_excess_odd

   if nscans gt 2 then begin
      signal(ind_scans(0):ind_scans(1)-1, *) = rebin(simul_excess_even, nt_scan_even, nb)
      save, file=file_signal, signal, /xdr
      prepare_proj_fine, file_scalars=file_scalars, scan_next=1, ind_scans=ind_scans
      proj_map, file_scalars=file_scalars, maxnoise=simnoise, ind_scans=ind_scans
      restore, file_maps
      map_excess_even = map_signal
      map_weight_even = map_weight
      signal(ind_scans(1):ind_scans(2)-1, *) = rebin(simul_excess_odd, nt_scan_odd, nb)
      save, file=file_signal, signal, /xdr
      prepare_proj_fine, file_scalars=file_scalars, scan_next=2, ind_scans=ind_scans
      proj_map, file_scalars=file_scalars, maxnoise=simnoise, ind_scans=ind_scans
      restore, file_maps
      map_excess_odd = map_signal
      map_weight_odd = map_weight

      if keyword_set(debug) then begin
         time_2 = systime(/seconds)
         t_proc += (time_2 - time_1)
         disp_ima, win=12, map_excess_even, weightmap=weight_drift(*, *, 0), $
            title='mapping of excess average drift in scan 0', min_map=min_d, max_map=max_d, noise_level=0.01, pow_int=0.5
         disp_ima, win=13, map_excess_odd, weightmap=weight_drift(*, *, 1), $
            title='mapping of excess average drift in scan 1', min_map=min_d, max_map=max_d, noise_level=0.01, pow_int=0.5
         res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
         wdelete, 10, 11, 12, 13
         time_1 = systime(/seconds)
      endif

      w = where(map_weight_even le 0.)
      map_excess_even(w) = !values.f_nan
      w = where(map_weight_odd le 0.)
      map_excess_odd(w) = !values.f_nan

      for i = 2, nscans - 1, 2 do simul_aver(ind_scans(i):ind_scans(i+1)-1) = $
         map_excess_even(indxy_bref(ind_scans(i):ind_scans(i+1)-1))
     for i = 3, nscans - 1, 2 do simul_aver(ind_scans(i):ind_scans(i+1)-1) = $
         map_excess_odd(indxy_bref(ind_scans(i):ind_scans(i+1)-1))
   endif else if keyword_set(debug) then wdelete, 10, 11
   if (keyword_set(debug) and nscans lt 8) then wdelete, 9


   x_bolos = 0
   y_bolos = 0
   wz = where(finite(simul_aver) lt 1, cwz, complement=wdef, ncomplement=cdef)
   if cwz ge 1 then begin
      tab_time = [time_s(0)-timestep, time_s(wdef), time_s(nt-1)+timestep]
      tab_drift = [simul_aver(wdef(0)), simul_aver(wdef), simul_aver(wdef(cdef-1))]
      simul_aver(wz) = interpol(tab_drift, tab_time, time_s(wz))
   endif
   simul_aver -= median(simul_aver)
   drifts -= simul_aver
   drifts_save -= interpol(simul_aver, time_s, time_save)

   signal = signal_save
   save, file=file_signal, signal, /xdr
   map_signal = map_signal_save
   map_weight = map_weight_save
   save, file=file_maps, map_signal, map_weight, map_error, map_lowfnoise, /xdr
   prepare_proj_fine, file_scalars=file_scalars, ind_scans=ind_scans


endif else begin
   restore, file_signal
   signal_save = signal
   restore, file_maps
   map_signal_save = map_signal
   map_weight_save = map_weight

   map_drift = fltarr(dimx, dimy, nsubscans)
   weight_drift = fltarr(dimx, dimy, nsubscans)
   signal = rebin(drifts, nt, nb)
   save, file=file_signal, signal, /xdr
   for i = 0L, nsubscans - 1L do begin
      prepare_proj_fine, file_scalars=file_scalars, subscan_next=i+1, ind_scans=ind_scans, ind_subscans=ind_subscans
      proj_map, file_scalars=file_scalars, maxnoise=median(maxnoise)+fltarr(nscans, nb), ind_scans=ind_scans
      restore, file_maps
      map_drift(*, *, i) = map_signal
      weight_drift(*, *, i) = map_weight
   endfor

   if keyword_set(visu) then begin
      time_2 = systime(/seconds)
      t_proc += (time_2 - time_1)
      disp_ima, win=10, map_drift(*, *, 0), title='mapping of subscan 0 average drift'
      disp_ima, win=11, map_drift(*, *, 1), title='mapping of subscan 1 average drift'
      res = dialog_message('   Continue ?   ', /center, /question)
      time_1 = systime(/seconds)
   endif

   dimxy = dimx * dimy
   restore, file_pixcoords
   for s = 0, nscans - 1 do begin
      wsubs = where(ind_subscans ge ind_scans(s) and ind_subscans lt ind_scans(s+1), csubs)
      simul_aver_2 = fltarr(ind_subscans(wsubs(1))-ind_subscans(wsubs(0)))
      for i = 0L, csubs - 2L do begin
         map_aver = fltarr(dimx, dimy) + !values.f_nan
         w = where(weight_drift(*, *, wsubs(i)) gt 0. and weight_drift(*, *, wsubs(i+1)) gt 0.)
         map_aver(w) = (map_drift(w+wsubs(i)*dimxy) + map_drift(w+wsubs(i+1)*dimxy)) / 2.

         simul_aver_1 = map_aver(floor(x_bolos(ind_subscans(wsubs(i)):ind_subscans(wsubs(i)+1)-1, *)) $
            + dimx * floor(y_bolos(ind_subscans(wsubs(i)):ind_subscans(wsubs(i)+1)-1, *)))
         simul_aver_1 = median(simul_aver_1, dimension=2)
         wz = where(finite(simul_aver_1) lt 1, cwz, complement=wdef, ncomplement=cdef)
         if cwz ge 1 then begin
            tab_time = [time_s(ind_subscans(wsubs(i)))-timestep, time_s(ind_subscans(wsubs(i))+wdef), $
               time_s(ind_subscans(wsubs(i)+1)-1)+timestep]
            tab_drift = [simul_aver_1(wdef(0)), simul_aver_1(wdef), simul_aver_1(wdef(cdef-1))]
            simul_aver_1(wz) = interpol(tab_drift, tab_time, time_s(ind_subscans(wsubs(i))+wz))
         endif
         simul_aver_1 -= simul_aver_2
         drifts(ind_subscans(wsubs(i)):ind_subscans(wsubs(i)+1)-1) -= simul_aver_1
         wsave = where(time_save ge time_s(ind_subscans(wsubs(i))) $
            and time_save le time_s(ind_subscans(wsubs(i)+1)-1))
         drifts_save(wsave) -= interpol(simul_aver_1, $
            time_s(ind_subscans(wsubs(i)):ind_subscans(wsubs(i)+1)-1), time_save(wsave))

         simul_aver_2 = map_aver(floor(x_bolos(ind_subscans(wsubs(i)+1):ind_subscans(wsubs(i)+2)-1, *)) $
            + dimx * floor(y_bolos(ind_subscans(wsubs(i)+1):ind_subscans(wsubs(i)+2)-1, *)))
         simul_aver_2 = median(simul_aver_2, dimension=2)
         wz = where(finite(simul_aver_2) lt 1, cwz, complement=wdef, ncomplement=cdef)
         if cwz ge 1 then begin
            tab_time = [time_s(ind_subscans(wsubs(i)+1))-timestep, time_s(ind_subscans(wsubs(i)+1)+wdef), $
               time_s(ind_subscans(wsubs(i)+2)-1)+timestep]
            tab_drift = [simul_aver_2(wdef(0)), simul_aver_2(wdef), simul_aver_2(wdef(cdef-1))]
            simul_aver_2(wz) = interpol(tab_drift, tab_time, time_s(ind_subscans(wsubs(i+1))+wz))
         endif
         drifts(ind_subscans(wsubs(i)+1):ind_subscans(wsubs(i)+2)-1) -= simul_aver_2
         wsave = where(time_save ge time_s(ind_subscans(wsubs(i)+1)) $
            and time_save le time_s(ind_subscans(wsubs(i)+2)-1))
         drifts_save(wsave) -= interpol(simul_aver_2, $
            time_s(ind_subscans(wsubs(i)+1):ind_subscans(wsubs(i)+2)-1), time_save(wsave))
      endfor
   endfor
   x_bolos = 0
   y_bolos = 0

   signal = signal_save
   save, file=file_signal, signal, /xdr
   map_signal = map_signal_save
   map_weight = map_weight_save
   save, file=file_maps, map_signal, map_weight, map_error, map_lowfnoise, /xdr
   prepare_proj_fine, file_scalars=file_scalars, ind_scans=ind_scans
endelse



w = where(npairs_save gt npairs_min)
med = median(drifts_save(w))
for i = 0, nscans - 1 do begin
   w = where(flag_nza(ind_scans(i):ind_scans(i+1)-1) gt 0)
   dev = sqrt(mean((drifts(ind_scans(i)+w) - med)^2.))
   amp_drifts(i, *) = 3. * dev
endfor



restore, file_signal


;;; visualize result:

if keyword_set(visu) then begin
   time_2 = systime(/seconds)
   t_proc += (time_2 - time_1)
   window, 0, xs=1200
   w = where(finite(signal) lt 1, cw)
   mask_bolo = 1 + intarr(nt, nb)
   if cw ge 1 then mask_bolo(w) = 0
   signal_aver = total(signal * mask_bolo, 2, /nan)
   norm = total(mask_bolo, 2)
   mask_bolo = 0
   wdef = where(norm gt nb / 5, complement=wz, ncomplement=nz)
   signal_aver(wdef) /= norm(wdef)
   if nz ge 1 then signal_aver(wz) = !values.f_nan
   med = median(signal_aver(wdef))
   dev = sqrt(mean((signal_aver(wdef) - med)^2.))
   for k = 0, 2 do begin
      wdev = where(norm gt nb / 5. and abs(signal_aver - med) lt 6. * dev)
      med = median(signal_aver(wdev))
      dev = sqrt(mean((signal_aver(wdev) - med)^2.))
   endfor
   norm = 0
   yrange = [-5.*dev, 6.*dev]
   otherplot, time_s(wdef), signal_aver(wdef) - med, charsize=1.5, yrange=yrange, $
      title=title, xtitle='t (s)', ytitle='brightness + offset'
   w = where(npairs_save gt 0)
   otherplot, /over, time_save(w), drifts_save(w) - 3. * dev, psym=4, color=10
   otherplot, /over, time_s, drifts - 3. * dev, color=100
   if nsubscans le 40 then begin
      otherplot, /over, time_s, dev * (13.8 * flag_nza - 10.), color=200
      for subs = 0L, nsubscans - 1 do otherplot, /over, time_s(ind_subscans(subs))+[0.,0.], yrange, color=30, lstyle=2.
   endif
   for i = 0, nscans - 2 do begin
      otherplot, /over, color=200, lstyle=2., time_s(ind_scans(i+1)-1)+fltarr(2), yrange
      otherplot, /over, color=200, lstyle=2., time_s(ind_scans(i+1))+fltarr(2), yrange
   endfor
   res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
   time_1 = systime(/seconds)
endif



print, '              drifts amplitude / noise (median, min and max) = ' $
   + chain(median(amp_drifts / maxnoise), 2) + ' ; ' $
   + chain(min(amp_drifts / maxnoise, /nan), 2) + ' to ' + chain(max(amp_drifts / maxnoise, /nan), 2) + crs



;;; update signal and total drifts:

if keyword_set(visu) then begin
   time_2 = systime(/seconds)
   t_proc += (time_2 - time_1)
   res = dialog_message('Subtract average drift ?', /center, /question)
   time_1 = systime(/seconds)
endif else res = 'Yes'

if res eq 'Yes' then begin
   signal -= rebin(drifts, nt, nb)
   drift_thermal += drifts_save
   drifts_all += rebin(drifts_save, nt_save, nb)
   save, file=file_signal, signal, /xdr

   ;;; visualization of drift:
   if keyword_set(visu) then begin
      time_2 = systime(/seconds)
      t_proc += (time_2 - time_1)
      restore, file_maps
      proj_map, map_nodrifts, file_scalars=file_scalars, maxnoise=maxnoise, ind_scans=ind_scans
      title = 'map of iteration ' + chain(iter) + ' drifts'
      if keyword_set(detect_gl) then title = title + ' (+ glitches)'
      disp_ima, map_signal - map_nodrifts, win=3, title=title, $
         weightmap=map_weight, noise_level=median(maxnoise)
      if keyword_set(debug) then res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
      time_1 = systime(/seconds)
   endif
endif else amp_drifts *= 0.


return, amp_drifts

end
